@@ -201,6 +201,7 @@ kevent_copyin(struct kqueue *kq, const struct kevent *src, int nchanges,
 
     dbg_printf("nchanges=%d nevents=%d", nchanges, nevents);
 
+    /* TODO: refactor, this has become convoluted to support EV_RECEIPT */
     for (nret = 0; nchanges > 0; src++, nchanges--) {
 
         if (kevent_copyin_one(kq, src) < 0) {
@@ -219,7 +220,7 @@ kevent_copyin(struct kqueue *kq, const struct kevent *src, int nchanges,
 err_path:
         if (nevents > 0) {
             memcpy(eventlist, src, sizeof(*src));
-            eventlist->data = errno;
+            eventlist->data = status;
             nevents--;
             eventlist++;
             nret++;
