@@ -45,11 +45,17 @@ static void
 signal_handler(int sig)
 {
     struct sentry *s = &sigtbl[sig];
-    ssize_t n;
 
     dbg_printf("caught sig=%d", sig);
     atomic_inc(&s->s_cnt);
-    n = write(s->s_filt->kf_wfd, &sig, sizeof(sig));//FIXME:errhandling
+#if defined(__sun__)
+    if (port_send(s->s_filt->kf_kqueue->kq_port, 
+                   X_PORT_SOURCE_SIGNAL, &sigtbl[sig]) < 0) {
+        return; //FIXME: errorhandling
+    }
+#else
+    (void)write(s->s_filt->kf_wfd, &sig, sizeof(sig));//FIXME:errhandling
+#endif
 }
 
 static int
@@ -160,11 +166,17 @@ evfilt_signal_copyout(struct filter *filt,
     struct sentry *s;
     struct knote *kn;
     int sig;
-    ssize_t n;
 
-    n = read(filt->kf_pfd, &sig, sizeof(sig));//FIXME:errhandling
-    dbg_printf("got sig=%d", sig);
+#if defined(__sun__)
+    port_event_t *pe = (port_event_t *) pthread_getspecific(filt->kf_kqueue->kq_port_event);
+
+    s = (struct sentry *) pe->portev_user;
+    sig = s - &sigtbl[0];
+#else
+    read(filt->kf_pfd, &sig, sizeof(sig));//FIXME:errhandling
     s = &sigtbl[sig];
+#endif
+
     kn = s->s_knote;
     //TODO: READ counter: s->s_knote->kev.data = ?;
     /* TODO: dst->data should be the number of times the signal occurred */
